@@ -1,7 +1,8 @@
-from genericpath import exists
+import asyncio
 import os
 from pywebvue.utils import read_file_as_base64_url
 from trame.html import Span, vuetify, Triggers
+from trame.utils import ClientWindowProcess, AppServerThread, compose_callbacks
 
 import pywebvue
 import trame as tr
@@ -103,6 +104,68 @@ class AbstractLayout:
 
         _app.run_server(port=port)
 
+    def start_thread(
+        self, port=None, print_server_info=False, on_server_listening=None, **kwargs
+    ):
+        _app = tr.get_app_instance()
+        _app.name = self.name
+        _app.layout = self.html
+
+        if self.favicon:
+            _app.favicon = self.favicon
+
+        if print_server_info:
+            _app.on_ready = tr.print_server_info(
+                compose_callbacks(self.on_ready, on_server_listening)
+            )
+        else:
+            _app.on_ready = compose_callbacks(self.on_ready, on_server_listening)
+
+        # Dev validation
+        tr.validate_key_names()
+        server_thread = AppServerThread(_app, port, **kwargs)
+        server_thread.start()
+        return server_thread
+
+    def start_desktop_window(self, on_msg=None, **kwargs):
+        from multiprocessing import Queue
+
+        _msg_queue = Queue()
+
+        _app = tr.get_app_instance()
+        _app.name = self.name
+        _app.layout = self.html
+
+        async def process_msg():
+            keep_processing = True
+            while keep_processing:
+                await asyncio.sleep(0.5)
+                if not _msg_queue.empty():
+                    msg = _msg_queue.get_nowait()
+                    if on_msg:
+                        on_msg(msg)
+                    if msg == "closing":
+                        keep_processing = False
+                        _app.stop_server()
+
+        asyncio.get_event_loop().create_task(process_msg())
+
+        if self.favicon:
+            _app.favicon = self.favicon
+
+        def start_client(**_):
+            client_process = ClientWindowProcess(
+                title=_app.name, port=_app.server_port, msg_queue=_msg_queue, **kwargs
+            )
+            client_process.start()
+
+        _app.on_ready = compose_callbacks(self.on_ready, start_client)
+
+        # Dev validation
+        tr.validate_key_names()
+
+        _app.run_server(port=0)
+
 
 class FullScreenPage(AbstractLayout):
     """
@@ -234,3 +297,18 @@ class SinglePageWithDrawer(SinglePage):
         self.toolbar.clipped_left = True
         self.children += [self.drawer]
         self.logo.click = f"{show_drawer_name} = !{show_drawer_name}"
+
+
+def update_layout(layout):
+    """
+    Flush layout to the client
+
+    :param layout: UI content for your application
+    :type layout: str | trame.layouts.*
+
+    >>> layout.title.set_text("Workload finished!")
+    >>> update_layout(layout)
+
+    """
+    _app = tr.get_app_instance()
+    _app.layout = layout if isinstance(layout, str) else layout.html
